/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.vew.engvew.engstelle;

import static org.junit.Assert.assertTrue;

import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.vew.engvew.testtools.InputData;
import de.bsvrz.vew.engvew.testtools.JUnitHelper;


public class TestBaustelleEngpass
{

  private static SystemObject _sysObjBaustelle = null;
  
  
  private static List<InputData> _baustellenData = new LinkedList<InputData>();
  
  private static List<InputData> _baustellenDataAS = new LinkedList<InputData>();
  private static List<InputData> _baustellenDataAK = new LinkedList<InputData>();
  
  private static SystemObject _mq;
  private static SystemObject _mqAS;
  private static SystemObject _mqAK;
      
  @BeforeClass
  public static void setUpBeforeClass() throws Exception 
  {
    _sysObjBaustelle = JUnitHelper.getInstance().erzeugeBaustelle();
          
    DataModel dm = JUnitHelper.getInstance().getConnection().getDataModel();
    Aspect asp = dm.getAspect("asp.parameterSoll");
    AttributeGroup atg = dm.getAttributeGroup("atg.situationsEigenschaften"); 
    DataDescription ddSituation =  new DataDescription(atg, asp);
    atg = dm.getAttributeGroup("atg.baustellenEigenschaften"); 
    DataDescription ddBaustelle =  new DataDescription(atg, asp);
    atg = dm.getAttributeGroup("atg.engstellenVerwaltungParameter");
    DataDescription ddParam =  new DataDescription(atg, asp);
  
    long aktuelleZeit = JUnitHelper.getInstance().getConnection().getTime();
   
    _mq = dm.getObject("mq.bw.mq_6_hfb1_sw");
    SystemObject ass = dm.getObject("ass.p.11907.p.11908.0");
   
    _mqAS = dm.getObject("mq.bw.mq_7_hfb1_sw");
    SystemObject assAS1 = dm.getObject("ass.p.11908.p.11909.0");
    SystemObject assAS2 = dm.getObject("ass.p.11907.p.11908.0");
    
    _mqAK = dm.getObject("mq.bw.mq_8_hfb1_sw");
    SystemObject assAK1 = dm.getObject("ass.p.11909.p.11910.0");
    SystemObject assAK2 = dm.getObject("ass.p.11908.p.11909.0");
    
    ResultData situation, baustelle, parameter;
    
    /* Beginn Testflle - normal - ohne AS, ohne AK */
    JUnitHelper.getInstance().setzeVerkehrstaerkeMq(_mq, 1300);
    
    /* 1. Testfall:
     * Baustelle hat Restkapazitt von 1050 Fzg/h  - Faktor ist 1.3 */
    situation = new ResultData(_sysObjBaustelle, ddSituation, aktuelleZeit, 
        JUnitHelper.getInstance().getDataSituationsEigenschaften(aktuelleZeit, 0,
        new SystemObject[] { ass }, 50, 450));
    baustelle = new ResultData(_sysObjBaustelle, ddBaustelle, aktuelleZeit, 
        JUnitHelper.getInstance().getDataBaustellenEigenschaften("gltig", 1050));
    parameter = new ResultData(JUnitHelper.getInstance().getEngVew().getObjekt(), ddParam, aktuelleZeit,
        JUnitHelper.getInstance().getDataFaktorMaxVerkehrsStaerke(1.3f)); // Faktor ist 1.3
    _baustellenData.add(new InputData(parameter, situation, baustelle, null, null));
    
    /* 2. Testfall:
     * Baustelle hat Restkapazitt von 950 Fzg/h  - Faktor ist 1.3 */
    situation = new ResultData(_sysObjBaustelle, ddSituation, aktuelleZeit, 
        JUnitHelper.getInstance().getDataSituationsEigenschaften(aktuelleZeit, 0,
        new SystemObject[] { ass }, 50, 450));
    baustelle = new ResultData(_sysObjBaustelle, ddBaustelle, aktuelleZeit, 
        JUnitHelper.getInstance().getDataBaustellenEigenschaften("gltig", 950));
    parameter = new ResultData(JUnitHelper.getInstance().getEngVew().getObjekt(), ddParam, aktuelleZeit,
        JUnitHelper.getInstance().getDataFaktorMaxVerkehrsStaerke(1.3f)); // Faktor ist 1.3
    _baustellenData.add(new InputData(parameter, situation, baustelle, null, null));

    /* 3. Testfall:
     * Baustelle hat Restkapazitt von 1050 Fzg/h - Faktor ist 1.2 */
    situation = new ResultData(_sysObjBaustelle, ddSituation, aktuelleZeit, 
        JUnitHelper.getInstance().getDataSituationsEigenschaften(aktuelleZeit, 0,
        new SystemObject[] { ass }, 50, 450));
    baustelle = new ResultData(_sysObjBaustelle, ddBaustelle, aktuelleZeit, 
        JUnitHelper.getInstance().getDataBaustellenEigenschaften("gltig", 1050));
    parameter = new ResultData(JUnitHelper.getInstance().getEngVew().getObjekt(), ddParam, aktuelleZeit,
        JUnitHelper.getInstance().getDataFaktorMaxVerkehrsStaerke(1.2f)); // Faktor ist 1.2
    _baustellenData.add(new InputData(parameter, situation, baustelle, null, null));
    
    /* 4. Testfall:
     * Baustelle hat Restkapazitt von 950 Fzg/h - Faktor ist 1.4 */
    situation = new ResultData(_sysObjBaustelle, ddSituation, aktuelleZeit, 
        JUnitHelper.getInstance().getDataSituationsEigenschaften(aktuelleZeit, 0,
        new SystemObject[] { ass }, 50, 450));
    baustelle = new ResultData(_sysObjBaustelle, ddBaustelle, aktuelleZeit, 
        JUnitHelper.getInstance().getDataBaustellenEigenschaften("gltig", 950));
    parameter = new ResultData(JUnitHelper.getInstance().getEngVew().getObjekt(), ddParam, aktuelleZeit,
        JUnitHelper.getInstance().getDataFaktorMaxVerkehrsStaerke(1.4f)); // Faktor ist 1.4
    _baustellenData.add(new InputData(parameter, situation, baustelle, null, null));
    
    /* Ende Testflle - normal - ohne AS, ohne AK */
    
    /* Beginn Testflle mit AS */
    JUnitHelper.getInstance().setzeVerkehrstaerkeMq(_mqAS, 1300);
    
    /* 1. Testfall:
     * Baustelle hat Restkapazitt von 950 Fzg/h  - Faktor ist 1.3 */
    situation = new ResultData(_sysObjBaustelle, ddSituation, aktuelleZeit, 
        JUnitHelper.getInstance().getDataSituationsEigenschaften(aktuelleZeit, 0,
        new SystemObject[] { assAS1 }, 50, 450));
    baustelle = new ResultData(_sysObjBaustelle, ddBaustelle, aktuelleZeit, 
        JUnitHelper.getInstance().getDataBaustellenEigenschaften("gltig", 950));
    parameter = new ResultData(JUnitHelper.getInstance().getEngVew().getObjekt(), ddParam, aktuelleZeit,
        JUnitHelper.getInstance().getDataFaktorMaxVerkehrsStaerke(1.3f)); // Faktor ist 1.3
    _baustellenDataAS.add(new InputData(parameter, situation, baustelle, null, null));

    /* 2. Testfall:
     * Baustelle hat Restkapazitt von 1050 Fzg/h - Faktor ist 1.2 */
    situation = new ResultData(_sysObjBaustelle, ddSituation, aktuelleZeit, 
        JUnitHelper.getInstance().getDataSituationsEigenschaften(aktuelleZeit, 0,
        new SystemObject[] { assAS1 }, 50, 450));
    baustelle = new ResultData(_sysObjBaustelle, ddBaustelle, aktuelleZeit, 
        JUnitHelper.getInstance().getDataBaustellenEigenschaften("gltig", 1050));
    parameter = new ResultData(JUnitHelper.getInstance().getEngVew().getObjekt(), ddParam, aktuelleZeit,
        JUnitHelper.getInstance().getDataFaktorMaxVerkehrsStaerke(1.2f)); // Faktor ist 1.2
    _baustellenDataAS.add(new InputData(parameter, situation, baustelle, null, null));

    /* 3. Testfall:
     * Baustelle hat Restkapazitt von 950 Fzg/h  - Faktor ist 1.3 
     * gleich wie 1. Testfall, zwischen BAustelle und MQ ist eine AS */
    situation = new ResultData(_sysObjBaustelle, ddSituation, aktuelleZeit, 
        JUnitHelper.getInstance().getDataSituationsEigenschaften(aktuelleZeit, 0,
        new SystemObject[] { assAS2 }, 1550, 1950));
    baustelle = new ResultData(_sysObjBaustelle, ddBaustelle, aktuelleZeit, 
        JUnitHelper.getInstance().getDataBaustellenEigenschaften("gltig", 950));
    parameter = new ResultData(JUnitHelper.getInstance().getEngVew().getObjekt(), ddParam, aktuelleZeit,
        JUnitHelper.getInstance().getDataFaktorMaxVerkehrsStaerke(1.3f)); // Faktor ist 1.3
    _baustellenDataAS.add(new InputData(parameter, situation, baustelle, null, null));

    /* 4. Testfall:
     * Baustelle hat Restkapazitt von 1050 Fzg/h - Faktor ist 1.2 
     * gleich wie 21. Testfall, zwischen BAustelle und MQ ist eine AS */
    situation = new ResultData(_sysObjBaustelle, ddSituation, aktuelleZeit, 
        JUnitHelper.getInstance().getDataSituationsEigenschaften(aktuelleZeit, 0,
        new SystemObject[] { assAS2 }, 1550, 1950));
    baustelle = new ResultData(_sysObjBaustelle, ddBaustelle, aktuelleZeit, 
        JUnitHelper.getInstance().getDataBaustellenEigenschaften("gltig", 1050));
    parameter = new ResultData(JUnitHelper.getInstance().getEngVew().getObjekt(), ddParam, aktuelleZeit,
        JUnitHelper.getInstance().getDataFaktorMaxVerkehrsStaerke(1.2f)); // Faktor ist 1.2
    _baustellenDataAS.add(new InputData(parameter, situation, baustelle, null, null));
    
    /* Ende Testflle mit AS */
    
    /* Beginn Testflle mit AK */
    JUnitHelper.getInstance().setzeVerkehrstaerkeMq(_mqAK, 1300);
    
    /* 1. Testfall:
     * Baustelle hat Restkapazitt von 950 Fzg/h  - Faktor ist 1.3 */
    situation = new ResultData(_sysObjBaustelle, ddSituation, aktuelleZeit, 
        JUnitHelper.getInstance().getDataSituationsEigenschaften(aktuelleZeit, 0,
        new SystemObject[] { assAK1 }, 50, 450));
    baustelle = new ResultData(_sysObjBaustelle, ddBaustelle, aktuelleZeit, 
        JUnitHelper.getInstance().getDataBaustellenEigenschaften("gltig", 950));
    parameter = new ResultData(JUnitHelper.getInstance().getEngVew().getObjekt(), ddParam, aktuelleZeit,
        JUnitHelper.getInstance().getDataFaktorMaxVerkehrsStaerke(1.3f)); // Faktor ist 1.3
    _baustellenDataAK.add(new InputData(parameter, situation, baustelle, null, null));

    /* 2. Testfall:
     * Baustelle hat Restkapazitt von 1050 Fzg/h - Faktor ist 1.2 */
    situation = new ResultData(_sysObjBaustelle, ddSituation, aktuelleZeit, 
        JUnitHelper.getInstance().getDataSituationsEigenschaften(aktuelleZeit, 0,
        new SystemObject[] { assAK1 }, 50, 450));
    baustelle = new ResultData(_sysObjBaustelle, ddBaustelle, aktuelleZeit, 
        JUnitHelper.getInstance().getDataBaustellenEigenschaften("gltig", 1050));
    parameter = new ResultData(JUnitHelper.getInstance().getEngVew().getObjekt(), ddParam, aktuelleZeit,
        JUnitHelper.getInstance().getDataFaktorMaxVerkehrsStaerke(1.2f)); // Faktor ist 1.2
    _baustellenDataAK.add(new InputData(parameter, situation, baustelle, null, null));

    /* 3. Testfall:
     * Baustelle hat Restkapazitt von 950 Fzg/h  - Faktor ist 1.3 
     * gleich wie 1. Testfall, zwischen BAustelle und MQ ist eine AS */
    situation = new ResultData(_sysObjBaustelle, ddSituation, aktuelleZeit, 
        JUnitHelper.getInstance().getDataSituationsEigenschaften(aktuelleZeit, 0,
        new SystemObject[] { assAK2 }, 1550, 1950));
    baustelle = new ResultData(_sysObjBaustelle, ddBaustelle, aktuelleZeit, 
        JUnitHelper.getInstance().getDataBaustellenEigenschaften("gltig", 950));
    parameter = new ResultData(JUnitHelper.getInstance().getEngVew().getObjekt(), ddParam, aktuelleZeit,
        JUnitHelper.getInstance().getDataFaktorMaxVerkehrsStaerke(1.3f)); // Faktor ist 1.3
    _baustellenDataAK.add(new InputData(parameter, situation, baustelle, null, null));

    /* 4. Testfall:
     * Baustelle hat Restkapazitt von 1050 Fzg/h - Faktor ist 1.2 
     * gleich wie 21. Testfall, zwischen BAustelle und MQ ist eine AS */
    situation = new ResultData(_sysObjBaustelle, ddSituation, aktuelleZeit, 
        JUnitHelper.getInstance().getDataSituationsEigenschaften(aktuelleZeit, 0,
        new SystemObject[] { assAK2 }, 1550, 1950));
    baustelle = new ResultData(_sysObjBaustelle, ddBaustelle, aktuelleZeit, 
        JUnitHelper.getInstance().getDataBaustellenEigenschaften("gltig", 1050));
    parameter = new ResultData(JUnitHelper.getInstance().getEngVew().getObjekt(), ddParam, aktuelleZeit,
        JUnitHelper.getInstance().getDataFaktorMaxVerkehrsStaerke(1.2f)); // Faktor ist 1.2
    _baustellenDataAK.add(new InputData(parameter, situation, baustelle, null, null));
    
    /* Ende Testflle mit AK */
    
    JUnitHelper.getInstance().getConnection().sleep(JUnitHelper.DELAY_INIT);
  }

  @AfterClass
  public static void tearDownAfterClass() throws Exception 
  {
    JUnitHelper.getInstance().loescheBaustelle(_sysObjBaustelle);
    _sysObjBaustelle = null;
    _baustellenData.clear();
    _baustellenDataAS.clear();
    _baustellenDataAK.clear();
    _mq = null;
    _mqAS = null;
    _mqAK = null;
  }
  
  @Before
  public void setUp() throws Exception 
  {
   
  }

  @After
  public void tearDown() throws Exception 
  {
  
  }

  @Test
  public final void testPruefeEngpass() 
  {
    Baustelle baustelle = null;
    for (Situation lauf: JUnitHelper.getInstance().getEngVew().getVerwalteteObjekte())
    {
      if (lauf.getObjekt().equals(_sysObjBaustelle))
      {
        baustelle = (Baustelle)lauf;
        break;
      }
    }
    assertTrue("Baustelle wird verwaltet", baustelle != null &&
        baustelle.getObjekt() == _sysObjBaustelle);
       
    int laufendeTestNr = 0;
    
    for (InputData inputData: _baustellenData)
    {
      baustelle.update(new ResultData[] { inputData.situationsEigenschaften });
      baustelle.update(new ResultData[] { inputData.baustellenEigenschaften });
      JUnitHelper.getInstance().getEngVew().update(new ResultData[] { inputData.engstellenVerwaltung });
      
      JUnitHelper.getInstance().getConnection().sleep(JUnitHelper.DELAY_UPDATE);
      
      laufendeTestNr += 1;
      
      Collection<ErgebnisEngpassPruefung> result = baustelle.pruefeEngpass();
            
      if (laufendeTestNr == 1)
      {
        assertTrue("Prfung Engpass nicht verletzt", result.isEmpty());
      }
      if (laufendeTestNr == 2)
      {
        assertTrue("Verkehrsstrke hinter Baustelle bersteigt Kapazitt um zulssigen Faktor",
            result.size() == 1 &&
            result.iterator().next().mq == _mq &&
            result.iterator().next().rk == 950 &&
            result.iterator().next().vs == 1300);
      }
      if (laufendeTestNr == 3)
      {
        assertTrue("Verkehrsstrke hinter Baustelle bersteigt Kapazitt um zulssigen Faktor",
            result.size() == 1 &&
            result.iterator().next().mq == _mq &&
            result.iterator().next().rk == 1050 &&
            result.iterator().next().vs == 1300);
      }
      if (laufendeTestNr == 4)
      {
        assertTrue("Prfung Engpass nicht verletzt", result.isEmpty());
      }
      
      System.out.println("Engpassprfung Nr. " + laufendeTestNr + ": OK");
    }
  }
  
  @Test
  public final void testPruefeEngpassAS() 
  {
    Baustelle baustelle = null;
    for (Situation lauf: JUnitHelper.getInstance().getEngVew().getVerwalteteObjekte())
    {
      if (lauf.getObjekt().equals(_sysObjBaustelle))
      {
        baustelle = (Baustelle)lauf;
        break;
      }
    }
    assertTrue("Baustelle wird verwaltet", baustelle != null &&
        baustelle.getObjekt() == _sysObjBaustelle);
       
    int laufendeTestNr = 0;
    
    for (InputData inputData: _baustellenDataAS)
    {
      baustelle.update(new ResultData[] { inputData.situationsEigenschaften });
      baustelle.update(new ResultData[] { inputData.baustellenEigenschaften });
      JUnitHelper.getInstance().getEngVew().update(new ResultData[] { inputData.engstellenVerwaltung });
      
      JUnitHelper.getInstance().getConnection().sleep(JUnitHelper.DELAY_UPDATE);
      
      laufendeTestNr += 1;
      
      Collection<ErgebnisEngpassPruefung> result = baustelle.pruefeEngpass();
    
      if (laufendeTestNr == 1)
      {
        assertTrue("Verkehrsstrke hinter Baustelle bersteigt Kapazitt um zulssigen Faktor",
            result.size() == 1 &&
            result.iterator().next().mq == _mqAS &&
            result.iterator().next().rk == 950 &&
            result.iterator().next().vs == 1300);
      }
      if (laufendeTestNr == 2)
      {
        assertTrue("Verkehrsstrke hinter Baustelle bersteigt Kapazitt um zulssigen Faktor",
            result.size() == 1 &&
            result.iterator().next().mq == _mqAS &&
            result.iterator().next().rk == 1050 &&
            result.iterator().next().vs == 1300);
      }
      if (laufendeTestNr == 3)
      {
        assertTrue("Prfung Engpass nicht verletzt", result.isEmpty());
      }
      if (laufendeTestNr == 4)
      {
        assertTrue("Prfung Engpass nicht verletzt", result.isEmpty());
      }
      
      System.out.println("Engpassprfung (mit Anschlusstelle) Nr. " + laufendeTestNr + ": OK");
    }
  }
  
  @Test
  public final void testPruefeEngpassAK() 
  {
    Baustelle baustelle = null;
    for (Situation lauf: JUnitHelper.getInstance().getEngVew().getVerwalteteObjekte())
    {
      if (lauf.getObjekt().equals(_sysObjBaustelle))
      {
        baustelle = (Baustelle)lauf;
        break;
      }
    }
    assertTrue("Baustelle wird verwaltet", baustelle != null &&
        baustelle.getObjekt() == _sysObjBaustelle);
       
    int laufendeTestNr = 0;
    
    for (InputData inputData: _baustellenDataAK)
    {
      baustelle.update(new ResultData[] { inputData.situationsEigenschaften });
      baustelle.update(new ResultData[] { inputData.baustellenEigenschaften });
      JUnitHelper.getInstance().getEngVew().update(new ResultData[] { inputData.engstellenVerwaltung });
      
      JUnitHelper.getInstance().getConnection().sleep(JUnitHelper.DELAY_UPDATE);
      
      laufendeTestNr += 1;
      
      Collection<ErgebnisEngpassPruefung> result = baustelle.pruefeEngpass();
    
      if (laufendeTestNr == 1)
      {
        assertTrue("Verkehrsstrke hinter Baustelle bersteigt Kapazitt um zulssigen Faktor",
            result.size() == 1 &&
            result.iterator().next().mq == _mqAK &&
            result.iterator().next().rk == 950 &&
            result.iterator().next().vs == 1300);
      }
      if (laufendeTestNr == 2)
      {
        assertTrue("Verkehrsstrke hinter Baustelle bersteigt Kapazitt um zulssigen Faktor",
            result.size() == 1 &&
            result.iterator().next().mq == _mqAK &&
            result.iterator().next().rk == 1050 &&
            result.iterator().next().vs == 1300);
      }
      if (laufendeTestNr == 3)
      {
        assertTrue("Prfung Engpass nicht verletzt", result.isEmpty());
      }
      if (laufendeTestNr == 4)
      {
        assertTrue("Prfung Engpass nicht verletzt", result.isEmpty());
      }
      
      System.out.println("Engpassprfung (mit Autobahnkreuz) Nr. " + laufendeTestNr + ": OK");
    }
  }
  
}
